import 'dart:html';
import 'dart:typed_data';
import 'dart:math' as Math;

var canvas;
var logDiv;
var image;

int _clamp255(int c) => (c < 0) ? 0 : (c > 255) ? 255 : c;

void adjustColor(Uint8List pixels, {int blacks, int whites, int mids,
                  double contrast, double saturation, double brightness,
                  double gamma, double exposure, double hue,
                  double amount}) {
  if (amount == 0.0) {
    return;
  }

  Float32List gammaLut;
  if (gamma != null) {
    gammaLut = _adjustColor_getLut(gamma);
  }

  const double DEG_TO_RAD = 0.0174532925;
  const double avgLumR = 0.5;
  const double avgLumG = 0.5;
  const double avgLumB = 0.5;
  const double lumCoeffR = 0.2125;
  const double lumCoeffG = 0.7154;
  const double lumCoeffB = 0.0721;

  Float32List mrLut;
  Float32List mgLut;
  Float32List mbLut;
  double br, bg, bb;
  double wr, wg, wb;
  double mr, mg, mb;
  /*if (blacks != null || whites != null || mids != null) {
    br = blacks != null ? getRed(blacks) / 255.0 : 0.0;
    bg = blacks != null ? getGreen(blacks) / 255.0 : 0.0;
    bb = blacks != null ? getBlue(blacks) / 255.0 : 0.0;

    wr = whites != null ? getRed(whites) / 255.0 : 1.0;
    wg = whites != null ? getGreen(whites) / 255.0 : 1.0;
    wb = whites != null ? getBlue(whites) / 255.0 : 1.0;

    mr = mids != null ? getRed(mids) / 255.0 : 0.5;
    mg = mids != null ? getGreen(mids) / 255.0 : 0.5;
    mb = mids != null ? getBlue(mids) / 255.0 : 0.5;

    mr = 1.0 / (1.0 + 2.0 * (mr - 0.5));
    mg = 1.0 / (1.0 + 2.0 * (mg - 0.5));
    mb = 1.0 / (1.0 + 2.0 * (mb - 0.5));

    mrLut = _adjustColor_getLut(mr);
    mgLut = _adjustColor_getLut(mg);
    mbLut = _adjustColor_getLut(mb);
  }*/

  double invSaturation = saturation != null ? 1.0 - saturation : 0.0;
  double invContrast = contrast != null ? 1.0 - contrast : 0.0;

  if (exposure != null) {
    exposure = Math.pow(2.0, exposure);
  }

  double hueR;
  double hueG;
  double hueB;
  if (hue != null) {
    hue *= DEG_TO_RAD;
    double s = Math.sin(hue);
    double c = Math.cos(hue);

    hueR = (2.0 * c) / 3.0;
    hueG = (-Math.sqrt(3.0) * s - c) / 3.0;
    hueB = ((Math.sqrt(3.0) * s - c) + 1.0) / 3.0;
  }

  double invAmount = amount != null ? 1.0 - amount : 0.0;

  for (int i = 0, len = pixels.length; i < len; i += 4) {
    double or = pixels[i] / 255.0;
    double og = pixels[i + 1] / 255.0;
    double ob = pixels[i + 2] / 255.0;

    double r = or;
    double g = og;
    double b = ob;

    /*if (br != null) {
      r = _adjustColor_lut((r + br) * wr, mrLut);
      g = _adjustColor_lut((g + bg) * wg, mgLut);
      b = _adjustColor_lut((b + bb) * wb, mbLut);*/
      /*r = Math.pow((r + br) * wr, mr);
      g = Math.pow((g + bg) * wg, mg);
      b = Math.pow((b + bb) * wb, mb);*/
    //}

    if (brightness != null && brightness != 1.0) {
      r *= brightness;
      g *= brightness;
      b *= brightness;
    }

    if (saturation != null) {
      double lum = r * lumCoeffR + g * lumCoeffG + b * lumCoeffB;

      r = lum * invSaturation + r * saturation;
      g = lum * invSaturation + g * saturation;
      b = lum * invSaturation + b * saturation;
    }

    if (contrast != null) {
      r = avgLumR * invContrast + r * contrast;
      g = avgLumG * invContrast + g * contrast;
      b = avgLumB * invContrast + b * contrast;
    }

    if (gamma != null) {
      r = _adjustColor_lut(r, gammaLut);
      g = _adjustColor_lut(g, gammaLut);
      b = _adjustColor_lut(b, gammaLut);
    }

    if (exposure != null) {
      r = r * exposure;
      g = g * exposure;
      b = b * exposure;
    }

    if (hue != null && hue != 0.0) {
      double hr = r * hueR + g * hueG + b * hueB;
      double hg = r * hueB + g * hueR + b * hueG;
      double hb = r * hueG + g * hueB + b * hueR;

      r = hr;
      g = hg;
      b = hb;
    }

    if (amount != null) {
      r = r * amount + or * invAmount;
      g = g * amount + og * invAmount;
      b = b * amount + ob * invAmount;
    }

    pixels[i] = _clamp255((r * 255.0).toInt());
    pixels[i + 1] = _clamp255((g * 255.0).toInt());
    pixels[i + 2] = _clamp255((b * 255.0).toInt());
  }
}

Map<num, Float32List> _adjustColor_luts = {};

_adjustColor_lut(double x, Float32List lut) =>
  lut[_clamp255((x * 255.0).toInt())];

Float32List _adjustColor_getLut(double y) {
  print('getLut $y');
  Float32List lut;
  if (_adjustColor_luts.containsKey(y)) {
    print('cached');
    lut = _adjustColor_luts[y];
  } else {
    lut = new Float32List(256);
    for (int i = 0; i < 256; ++i) {
      lut[i] = Math.pow(i / 255.0, y);
    }
    _adjustColor_luts[y] = lut;
  }
  return lut;
}

void _LOG(s) {
  DivElement d = new DivElement();
  d.text = s.toString();
  document.body.append(d);
}

void main() {
  canvas = document.querySelector('#filter_canvas');
  logDiv = document.querySelector('#log');

  double contrast = 1.0;
  double saturation = 0.75;
  double brightness = 1.3;
  double gamma = 1.5;
  double exposure = 0.0;
  double hue = 5.0;
  double amount = 1.0;

  ImageElement img = new ImageElement();
  img.src = 'big_buck_bunny.jpg';
  img.onLoad.listen((e) {
    var c = new CanvasElement();
    document.body.append(c);
    c.width = img.width;
    c.height = img.height;
    c.context2D.drawImage(img, 0, 0);

    var imageData = c.context2D.getImageData(0, 0, img.width, img.height);

    image = new Uint8List.view((imageData.data as TypedData).buffer);

    Stopwatch t = new Stopwatch();
    t.start();
    for (int i = 0; i < 10; ++i) {
      adjustColor(image, contrast: contrast, saturation: saturation,
              brightness: brightness, gamma: gamma, exposure: exposure,
              hue: hue, amount: amount);
    }
    t.stop();
    _LOG(t.elapsedMilliseconds / 1000.0);
  });
}
